//===-- lldb-private-forward.h ----------------------------------*- C++ -*-===//
//
// Part of the LLVM Project, under the Apache License v2.0 with LLVM Exceptions.
// See https://llvm.org/LICENSE.txt for license information.
// SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
//
//===----------------------------------------------------------------------===//

#ifndef LLDB_LLDB_PRIVATE_FORWARD_H
#define LLDB_LLDB_PRIVATE_FORWARD_H

namespace lldb_private {
// forward decls.
class NativeProcessProtocol;
class NativeRegisterContext;
class NativeThreadProtocol;
class ResumeActionList;
class UnixSignals;
}

#endif // LLDB_LLDB_PRIVATE_FORWARD_H
